<?php

namespace Drupal\commerce_promo_link\EventSubscriber;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Event\OrderEvent;
use Drupal\commerce_order\Event\OrderEvents;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class OrderEventSubscriber.
 *
 * This class provides a fix for the shipping cost calculation in the cart.
 * It covers the use case, that a customer is in the checkout and during the
 * checkout add additional products to the cart and expecting the shipping
 * cost to be displayed correctly on the cart.
 */
class OrderEventSubscriber implements EventSubscriberInterface {

  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Temp Store.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  protected $tempStore;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $currentRouteMatch;

  /**
   * Constructs a new CartEventSubscriber object.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    PrivateTempStoreFactory $temp_store,
    RouteMatchInterface $current_route_match
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->tempStore = $temp_store->get('commerce_promo_link');
    $this->currentRouteMatch = $current_route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [
      OrderEvents::ORDER_PRESAVE => ['onOrderUpdate', -100],
    ];
    return $events;
  }

  /**
   * Event listener for order update event.
   *
   * @param \Drupal\commerce_order\Event\OrderEvent $orderEvent
   *   The order event.
   */
  public function onOrderUpdate(OrderEvent $orderEvent) {
    $order = $orderEvent->getOrder();
    if (!empty($order)) {
      // Order needs order items and has to be a cart (=draft).
      if ($order->hasItems() && $order->getState()->getId() == 'draft') {
        $route_name = $this->currentRouteMatch->getRouteName();
        // Only react it a product was added to the cart.
        // Else the coupon code cannot be overridden in the checkout.
        $valid_routes = [
          'entity.node.canonical',
          'commerce_cart.page',
          'entity.commerce_product.canonical',
        ];
        if (in_array($route_name, $valid_routes)) {
          $this->applyCouponCode($order);
        }
      }
    }
  }

  /**
   * Apply coupon code.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function applyCouponCode(OrderInterface $order) {

    // The customer has to enter the checkout select a shipping
    // method before a shipping is saved to the database.
    $code = $this->tempStore->get('coupon_code');
    if (empty($code)) {
      return;
    }

    /** @var \Drupal\commerce_promotion\Entity\Coupon $coupon */
    $coupon = $this->entityTypeManager->getStorage('commerce_promotion_coupon')->loadEnabledByCode($code);
    if (!empty($coupon)) {
      $order->set('coupons', $coupon->id());
    }
    else {
      $this->messenger()->addWarning($this->t('Your coupon code @code has expired.', ['@code' => $code]));
    }

    $this->tempStore->delete('coupon_code');
  }

}
